#ifndef DISKMANAGER_H
#define DISKMANAGER_H

/*
   DiskManager.h - collection of helper routines to manage disk images
   and load/unload them into the SIO server (SIOManager)

   Copyright (C) 2003 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "SIOManager.h"
#include "AtrImage.h"
#include "RCPtr.h"
#include "RefCounted.h"

class DiskManager : public RefCounted {
public:
	DiskManager();
	virtual ~DiskManager();

	bool SetSioServerMode(SIOWrapper::ESIOServerCommandLine cmdLine);

	SIOWrapper::ESIOServerCommandLine GetSioServerMode() const;

	enum EDriveNumber {
		eNoDrive = -1,
		eAllDrives = 0,
		eDrive1 = 1,
		eDrive2 = 2,
		eDrive3 = 3,
		eDrive4 = 4,
		eDrive5 = 5,
		eDrive6 = 6,
		eDrive7 = 7,
		eDrive8 = 8,
		eMinDriveNumber = eDrive1,
		eMaxDriveNumber = eDrive8
	};

	enum {eSIODriveBase=0x30};

	// floppy disk functions:

	static RCPtr<DiskImage> LoadDiskImage(const char* filename, bool beQuiet = false);
	bool LoadDiskImage(EDriveNumber driveno, const char* filename, bool beQuiet = false);

	bool CreateAtrMemoryImage(EDriveNumber driveno, EDiskFormat format);
	bool CreateAtrMemoryImage(EDriveNumber driveno, ESectorLength density, size_t sectors);
	bool WriteDiskImage(EDriveNumber driveno, const char* filename);
	bool UnloadDiskImage(EDriveNumber driveno);
	bool SetWriteProtectImage(EDriveNumber driveno, bool on);
	bool WriteBackImagesIfChanged();
	const char* GetImageFilename(EDriveNumber driveno) const;
	bool ExchangeDrives(EDriveNumber drive1, EDriveNumber drive2);
	bool DriveInUse(EDriveNumber driveno) const;
	bool DriveIsWriteProtected(EDriveNumber driveno) const;
	bool DriveIsChanged(EDriveNumber driveno) const;
	size_t GetDriveImageSize(EDriveNumber driveno) const;
	bool DriveNumberOK(EDriveNumber driveno) const;

	enum EHighSpeedMode {
		eHighSpeedOff,
		eHighSpeedOn,
		eHighSpeedWithPause
	};

	bool SetHighSpeedMode(EHighSpeedMode mode);
	EHighSpeedMode GetHighSpeedMode() const;

	int DoServing(int otherReadPollDevice=-1);

	RCPtr<SIOManager> GetSIOManager();
	RCPtr<SIOWrapper> GetSIOWrapper();

	RCPtr<AtrImage> GetAtrImage(EDriveNumber driveno);
	RCPtr<const AtrImage> GetConstAtrImage(EDriveNumber driveno) const;
	// may return NULL

	RCPtr<DiskImage> GetDiskImage(EDriveNumber driveno);
	RCPtr<const DiskImage> GetConstDiskImage(EDriveNumber driveno) const;

	// returns true if there are any changed images
	bool CheckForChangedImages();

private:

	RCPtr<SIOWrapper> fSIOWrapper;
	RCPtr<SIOManager> fSIOManager;

	EHighSpeedMode fHighSpeedMode;
	bool fUseHighSpeed;
	SIOWrapper::ESIOServerCommandLine fCableType;
};

inline RCPtr<SIOManager> DiskManager::GetSIOManager()
{
	return fSIOManager;
}

inline RCPtr<SIOWrapper> DiskManager::GetSIOWrapper()
{
	return fSIOWrapper;
}

inline SIOWrapper::ESIOServerCommandLine DiskManager::GetSioServerMode() const
{
	return fCableType;
}

inline DiskManager::EHighSpeedMode DiskManager::GetHighSpeedMode() const
{
	return fHighSpeedMode;
}
#endif
