/*
   FileInput.cpp - extension of StringInput supporting filename completion

   Copyright (C) 2003 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "FileInput.h"
#include "SIOTracer.h"
#include <string.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <pwd.h>

#include "DirectoryCache.h"
#include "AtariDebug.h"

FileInput::FileInput(CursesFrontend* frontend, RCPtr<DirectoryCache>& dirCache)
	: super(frontend),
	  fDirCache(dirCache)
{
}

FileInput::~FileInput()
{
}

void FileInput::SplitInputIntoPathAndFilename()
{
	int sepPos = fEditPos - 1;
	while ( (sepPos >= 0) && (fString[sepPos] != '/') ) {
		sepPos--;
	}

	if (sepPos == -1) {
		strcpy(fPath,".");
		strncpy(fFilename, fString, NAME_MAX+1);
		if (fEditPos < NAME_MAX ) {
			fFilename[fEditPos] = 0;
		} else {
			fFilename[NAME_MAX] = 0;
		}
	} else {
		if (sepPos == 0) {
			strncpy(fPath,"/",PATH_MAX+1);
		} else {
			strncpy(fPath, fString, PATH_MAX+1);
			if (sepPos < PATH_MAX) {
				fPath[sepPos] = 0;
			} else {
				fPath[PATH_MAX] = 0;
			}
		}
		strncpy(fFilename, fString + sepPos + 1, NAME_MAX+1);
		if (fEditPos - sepPos - 1 < NAME_MAX) {
			fFilename[fEditPos - sepPos - 1] = 0;
		} else {
			fFilename[NAME_MAX] = 0;
		}
	}
	if ( (fPath[0]=='~') && ((fPath[1]=='/') || (fPath[1]==0)) ) {
		char *home = getenv("HOME");
		if (home == NULL) {
			struct passwd *pw = getpwuid(getuid());
			home = pw->pw_dir;
		}
		if (home != NULL) {
			unsigned int hlen = strlen(home);
			unsigned int plen = strlen(fPath);
			if (plen - 1 + hlen < PATH_MAX+1) {
				memmove(fPath + hlen - 1, fPath, plen);
				memcpy(fPath, home, hlen);
			}
		}
	}
}

bool FileInput::PrepareForFinish()
{
	if ( (fString[0]=='~') && (fString[1]=='/') ) {
		char *home = getenv("HOME");
		if (home == NULL) {
			struct passwd *pw = getpwuid(getuid());
			home = pw->pw_dir;
		}
		if (home != NULL) {
			char tmp[fCurrentStringLength + strlen(home)];
			strcpy(tmp, home);
			strcat(tmp, fString + 1);
			strncpy(fString, tmp, fMaxStringLength);
			fString[fMaxStringLength - 1] = 0;
		} else {
			return false;
		}
	}
	return true;
}


bool FileInput::TryComplete()
{
	if (fDirCache.IsNull()) {
		return false;
	}

	if ( (fEditPos == 1) && (fString[0] == '~') ) {
		return InsertChar('/');
	}
	SplitInputIntoPathAndFilename();

	RCPtr<Directory> dir = fDirCache->GetDirectory(fPath, true);

	if (dir.IsNull()) {
		return false;
	}

	unsigned int dirSize = dir->Size();
	unsigned int len = strlen(fFilename);
	char matchString[NAME_MAX+1];
	matchString[0] = 0;

	bool first = true;
	for (unsigned int num=0;num<dirSize;num++) {
		DirEntry* e = dir->Get(num);
		bool match = false;
		if (len == 0) {
			match = (e->fName[0] != '.');
		} else {
			match = (strncmp(fFilename, e->fName, len) == 0);
		}
		if (match) {
			if (first) {
				strncpy(matchString, e->fName, NAME_MAX+1);
				matchString[NAME_MAX] = 0;
				first = false;

				char fullPath[PATH_MAX+1];
				snprintf(fullPath, PATH_MAX+1, "%s/%s", fPath, e->fName);
				fullPath[PATH_MAX] = 0;

				struct stat statbuf;
				if (stat(fullPath, &statbuf) == 0) {
					if (S_ISDIR(statbuf.st_mode)) {
						strcat(matchString,"/");
					}
				}
			} else {
				int i = len;
				while (matchString[i] && (matchString[i] == e->fName[i])) {
					i++;
				}
				matchString[i] = 0;
			}
		}
	}

	if (strlen(matchString) > len) {
		if (!InsertString(matchString+len)) {
			return false;
		}
	} else {
		return false;
	}

	return true;
}

bool FileInput::PrepareForSelection()
{
	if (fClearOnFirstKey && fIsFirstKey) {
		fFilename[0] = 0;
		strcpy(fPath,".");
		return true;
	} else {
		SplitInputIntoPathAndFilename();
	}
	struct stat sb;
	if ( (stat(fPath,&sb) == 0) && (S_ISDIR(sb.st_mode)) ) {
		return true;
	}
	strcpy(fPath,".");

	return true;
}
